/*!
This file is part of CycloneDX JavaScript Library.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

SPDX-License-Identifier: Apache-2.0
Copyright (c) OWASP Foundation. All Rights Reserved.
*/
import type { Comparable } from '../_helpers/sortable';
import type { Stringable } from '../_helpers/stringable';
declare abstract class BomLinkBase implements Stringable, Comparable<Stringable> {
    #private;
    /**
     * @throws {@link RangeError} if value is invalid
     */
    constructor(value: string);
    /**
     * @throws {@link RangeError} if value is invalid
     */
    set value(value: string);
    get value(): string;
    compare(other: Stringable): number;
    toString(): string;
}
/**
 * Descriptor for another BOM document.
 *
 * See [the docs](https://cyclonedx.org/capabilities/bomlink/)
 */
export declare class BomLinkDocument extends BomLinkBase {
    #private;
    /**
     * Whether the `value` is a valid descriptor for another BOM document.
     */
    static isValid(value: any): boolean;
}
/**
 * Descriptor for an element in a BOM document.
 *
 * See [the docs](https://cyclonedx.org/capabilities/bomlink/)
 */
export declare class BomLinkElement extends BomLinkBase {
    #private;
    /**
     * Whether the `value` is a valid descriptor for an element in a BOM document.
     */
    static isValid(value: any): boolean;
}
/**
 * Either {@link BomLinkDocument} or {@link BomLinkElement}.
 *
 * See [the docs](https://cyclonedx.org/capabilities/bomlink/)
 */
export type BomLink = BomLinkDocument | BomLinkElement;
export {};
//# sourceMappingURL=bomLink.d.ts.map