/*!
This file is part of CycloneDX JavaScript Library.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

SPDX-License-Identifier: Apache-2.0
Copyright (c) OWASP Foundation. All Rights Reserved.
*/
/**
 * Details and information describing a validation error.
 */
export type ValidationError = NonNullable<any>;
export interface Validator {
    /**
     * Promise completes with one of the following:
     * - `null`, when data was valid
     * - {@link ValidationError | something} representing the error details, when data was invalid
     *
     * Promise rejects with one of the following:
     * - {@link Validation.NotImplementedError | NotImplementedError}, when there is no validator available
     * - {@link Validation.MissingOptionalDependencyError | MissingOptionalDependencyError}, when a required dependency was not installed
     */
    validate: (data: string) => Promise<null | ValidationError>;
}
//# sourceMappingURL=types.d.ts.map