/*!
This file is part of CycloneDX JavaScript Library.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

SPDX-License-Identifier: Apache-2.0
Copyright (c) OWASP Foundation. All Rights Reserved.
*/
import type { Comparable } from '../../_helpers/sortable';
import { SortableComparables } from '../../_helpers/sortable';
import { CweRepository } from '../../types';
import { BomRef } from '../bomRef';
import { PropertyRepository } from '../property';
import { ToolRepository } from '../tool';
import { AdvisoryRepository } from './advisory';
import { AffectRepository } from './affect';
import type { Analysis } from './analysis';
import type { Credits } from './credits';
import { RatingRepository } from './rating';
import { ReferenceRepository } from './reference';
import type { Source } from './source';
export interface OptionalVulnerabilityProperties {
    bomRef?: BomRef['value'];
    id?: Vulnerability['id'];
    source?: Vulnerability['source'];
    references?: Vulnerability['references'];
    ratings?: Vulnerability['ratings'];
    cwes?: Vulnerability['cwes'];
    description?: Vulnerability['description'];
    detail?: Vulnerability['detail'];
    recommendation?: Vulnerability['recommendation'];
    advisories?: Vulnerability['advisories'];
    created?: Vulnerability['created'];
    published?: Vulnerability['published'];
    updated?: Vulnerability['updated'];
    credits?: Vulnerability['credits'];
    tools?: Vulnerability['tools'];
    analysis?: Vulnerability['analysis'];
    affects?: Vulnerability['affects'];
    properties?: Vulnerability['properties'];
}
export declare class Vulnerability implements Comparable<Vulnerability> {
    #private;
    id?: string;
    source?: Source;
    references: ReferenceRepository;
    ratings: RatingRepository;
    cwes: CweRepository;
    description?: string;
    detail?: string;
    recommendation?: string;
    advisories: AdvisoryRepository;
    created?: Date;
    published?: Date;
    updated?: Date;
    credits?: Credits;
    tools: ToolRepository;
    analysis?: Analysis;
    affects: AffectRepository;
    properties: PropertyRepository;
    constructor(op?: OptionalVulnerabilityProperties);
    get bomRef(): BomRef;
    compare(other: Vulnerability): number;
}
export declare class VulnerabilityRepository extends SortableComparables<Vulnerability> {
}
//# sourceMappingURL=vulnerability.d.ts.map