"use strict";
/*!
This file is part of CycloneDX JavaScript Library.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

SPDX-License-Identifier: Apache-2.0
Copyright (c) OWASP Foundation. All Rights Reserved.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.AffectedVersionRepository = exports.AffectedVersionRange = exports.AffectedSingleVersion = exports.AffectRepository = exports.Affect = void 0;
const sortable_1 = require("../../_helpers/sortable");
class Affect {
    constructor(ref, op = {}) {
        this.ref = ref;
        this.versions = op.versions ?? new AffectedVersionRepository();
    }
    compare(other) {
        return this.ref.compare(other.ref) ||
            this.versions.compare(other.versions);
    }
}
exports.Affect = Affect;
class AffectRepository extends sortable_1.SortableComparables {
}
exports.AffectRepository = AffectRepository;
class AffectedSingleVersion {
    constructor(version, op = {}) {
        this.version = version;
        this.status = op.status;
    }
    compare(other) {
        return (this.version).localeCompare(other.version) ||
            (this.status ?? '').localeCompare(other.status ?? '');
    }
}
exports.AffectedSingleVersion = AffectedSingleVersion;
class AffectedVersionRange {
    constructor(range, op = {}) {
        this.range = range;
        this.status = op.status;
    }
    compare(other) {
        return (this.range).localeCompare(other.range) ||
            (this.status ?? '').localeCompare(other.status ?? '');
    }
}
exports.AffectedVersionRange = AffectedVersionRange;
class AffectedVersionRepository extends sortable_1.SortableComparables {
}
exports.AffectedVersionRepository = AffectedVersionRepository;
//# sourceMappingURL=affect.js.map