"use strict";
/*!
This file is part of CycloneDX JavaScript Library.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

SPDX-License-Identifier: Apache-2.0
Copyright (c) OWASP Foundation. All Rights Reserved.
*/
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Vulnerability_bomRef;
Object.defineProperty(exports, "__esModule", { value: true });
exports.VulnerabilityRepository = exports.Vulnerability = void 0;
const sortable_1 = require("../../_helpers/sortable");
const types_1 = require("../../types");
const bomRef_1 = require("../bomRef");
const property_1 = require("../property");
const tool_1 = require("../tool");
const advisory_1 = require("./advisory");
const affect_1 = require("./affect");
const rating_1 = require("./rating");
const reference_1 = require("./reference");
class Vulnerability {
    constructor(op = {}) {
        _Vulnerability_bomRef.set(this, void 0);
        __classPrivateFieldSet(this, _Vulnerability_bomRef, new bomRef_1.BomRef(op.bomRef), "f");
        this.id = op.id;
        this.source = op.source;
        this.references = op.references ?? new reference_1.ReferenceRepository();
        this.ratings = op.ratings ?? new rating_1.RatingRepository();
        this.cwes = op.cwes ?? new types_1.CweRepository();
        this.description = op.description;
        this.detail = op.detail;
        this.recommendation = op.recommendation;
        this.advisories = op.advisories ?? new advisory_1.AdvisoryRepository();
        this.created = op.created;
        this.published = op.published;
        this.updated = op.updated;
        this.credits = op.credits;
        this.tools = op.tools ?? new tool_1.ToolRepository();
        this.analysis = op.analysis;
        this.affects = op.affects ?? new affect_1.AffectRepository();
        this.properties = op.properties ?? new property_1.PropertyRepository();
    }
    get bomRef() {
        return __classPrivateFieldGet(this, _Vulnerability_bomRef, "f");
    }
    compare(other) {
        const bomRefCompare = this.bomRef.compare(other.bomRef);
        if (bomRefCompare !== 0) {
            return bomRefCompare;
        }
        return (this.id ?? '').localeCompare(other.id ?? '') ||
            (this.created?.getTime() ?? 0) - (other.created?.getTime() ?? 0) ||
            (this.published?.getTime() ?? 0) - (other.published?.getTime() ?? 0) ||
            (this.updated?.getTime() ?? 0) - (other.updated?.getTime() ?? 0) ||
            this.ratings.compare(other.ratings) ||
            this.cwes.compare(other.cwes) ||
            (this.description ?? '').localeCompare(other.description ?? '') ||
            (this.detail ?? '').localeCompare(other.detail ?? '') ||
            (this.recommendation ?? '').localeCompare(other.recommendation ?? '') ||
            (this.source && other.source ? this.source.compare(other.source) : 0) ||
            this.properties.compare(other.properties);
    }
}
exports.Vulnerability = Vulnerability;
_Vulnerability_bomRef = new WeakMap();
class VulnerabilityRepository extends sortable_1.SortableComparables {
}
exports.VulnerabilityRepository = VulnerabilityRepository;
//# sourceMappingURL=vulnerability.js.map